function ImageProcessing(n_start, n_end, FileGroup)
%
% ImageProcessing.m is part of the LCM project, published under the terms of
% GNU General Public License.
% 
% ******************************************
% 
%   Author: Jing Su <Jing.Su.66#gmail.com>
%   Copyright 2002-2009 Jing Su
%   Meredith Group, Georgia Tech
% 
% ******************************************
%
% This program processes
%   1. PreB, i.e., Counter staining, with 
%      - Removing Background
%      - washing
%   2. PreR, i.e., BrdU staining, with
%      - Removing Background
%   3. Ana
%   4. C2C Cal
% 
% For example:
%
% ImageProcessing(0, 395, 'Example')
% 
  
  
% Switches for the four processes 
  fPreB = 1;
  fPreR = 1;
  fAna = 1;

  t_start = cputime;
  tic;
    
    % Bypassing void images: based on meta-info, the index of void raw images
    % attained from scanning machine are recorded in FileGroup_BadImages.mat.
    % 
    % Void images may be collected. E.g., if a fixed scanning area is used for
    % chips of different sizes, images outside the chip boundaries may be
    % acquired for small chips. The index of void images can be obtained from
    % the meta-info of the scanning software, PolyScan, developed by Jing Su.
    % 

    if exist([FileGroup,'_BadImages.mat'], 'file'),
      load([FileGroup,'_BadImages']);
    else
      BadImages=[];
    end;

    for i = n_start:n_end,
      if any(BadImages==i), continue, end;

      %%%%%%%%%%%%%
      DataOut.i = i;
      if fPreB,
        %%%%%%%%%%%%%%%
        % Part I: PreB
        %%%%%%%%%%%%%%
        FileName = [FileGroup, '_B_10X_',int2str(i)];

        i_o = OpenImage([FileGroup, '_B_10X_'], i, '.jpg', FileGroup, 'gray');

        bg = imopen(i_o, strel('disk', 30));
        i_o = imsubtract(i_o,bg); % Removing background

        i_bw = im2bw(i_o, graythresh(i_o));
        FileNameOutBW = [FileName '-BW' '.jpg'];
        FullFileName = fullfile( ...
            'PreProcess', FileGroup, 'B', ...
            FileNameOutBW);
        imwrite(i_bw, FullFileName, 'JPEG');

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        SE3 = strel('disk',5);
        i_bw_fill = imfill(imopen(i_bw, SE3), 'holes');

        FileNameOut = [FileName '-Fill' '.jpg'];
        FullFileName = fullfile( ...
            'PreProcess', FileGroup, 'B', ...
            FileNameOut);
        imwrite(i_bw_fill, FullFileName, 'JPEG');

        %%%%Watershed
        D_QE = bwdist(~i_bw_fill, 'quasi-euclidean');
        D_QE = mat2gray(D_QE);
        f_a = fspecial('disk', 5);
        D_QE5 = imfilter(D_QE, f_a, 'replicate');
        D_QE5 = -D_QE5;
        D_QE5(~i_bw_fill) = -Inf;
        L_QE5 = watershed(D_QE5);
        i_mask = i_bw_fill .* L_QE5;
        %Remove particles smaller than 20 pixels
        [lf5,nf5] = bwlabel(i_mask);
        stats5 = regionprops(lf5,'all');
        idx = find([stats5.Area] <= 20);
        BW2 = ismember(lf5,idx);
        i_mask = i_mask .* (~BW2);
        
        FileNameOutBW = [FileName '-WS' '.jpg'];
        FullFileName = fullfile( ...
            'PreProcess', FileGroup, 'B', ...
            FileNameOutBW);
        imwrite(i_mask, FullFileName, 'JPEG');
        [int2str(i-n_start+1) ' of ' int2str(n_end-n_start+1) ' for PreB, time used: ' ...
         int2str(round(toc)) 'Sec.']
      end;

      %%%%%%%%%%%%%%%
      % Part II: PreR
      %%%%%%%%%%%%%%
      if fPreR,
        i_o_r = OpenImage([FileGroup, '_G_10X_'], i, '.jpg', FileGroup, 'gray');
        bg = imopen(i_o_r, strel('disk', 15));
        i_o_r = imsubtract(i_o_r,bg);
        
        [lf,nf] = bwlabel(i_mask);
        stats = regionprops(lf,'Area', 'Centroid','PixelIdxList');
        
        DataOut.N_Of_Cell = nf;
        DataOut.L_Cell = lf;
        DataOut.Stats_Cell = stats;
        
        int_d = zeros(size(stats, 1),1);
        int_s = zeros(size(stats, 1),1);
        
        for i_PreR = 1:size(stats, 1)
          int_d(i_PreR) = mean(i_o_r(stats(i_PreR).PixelIdxList));
          int_s(i_PreR) = sum(i_o_r(stats(i_PreR).PixelIdxList));
        end;

        level_s = isohist(int_s);
        level_d = isohist(int_d);
        
        i_prol_d = zeros(size(i_o_r));
        i_rest_d = zeros(size(i_o_r));
        
        i_prol_a = zeros(size(i_o_r));
        i_rest_a = zeros(size(i_o_r));

        for I_PreR = 1:size(stats, 1), 
          if int_d(I_PreR) >= level_d ,
            i_prol_d(stats(I_PreR).PixelIdxList) = 1;
          else,
            i_rest_d(stats(I_PreR).PixelIdxList) = 1;
          end;
          
          if int_d(I_PreR) >= level_d | int_s(I_PreR) >= level_s,
            i_prol_a(stats(I_PreR).PixelIdxList) = 1;
          else,
            i_rest_a(stats(I_PreR).PixelIdxList) = 1;
          end;
        end;

        rgb_prol_a = zeros([size(i_o_r) 3]);
        rgb_prol_a(:,:,1) = i_prol_a;
        rgb_prol_a(:,:,2) = i_rest_a;
        rgb_prol_a(:,:,3) = i_mask;

        rgb_prol_d = zeros([size(i_o_r) 3]);
        rgb_prol_d(:,:,1) = i_prol_d;
        rgb_prol_d(:,:,2) = i_rest_d;
        rgb_prol_d(:,:,3) = i_mask;
        
        rgb_prol_s = zeros([size(i_o_r) 3]); % screening: compare the difference
                                             % between the two method: d vs s
        rgb_prol_s(:,:,1) = i_prol_d;
        rgb_prol_s(:,:,2) = i_prol_a;
        rgb_prol_s(:,:,3) = i_mask;
        
        Dir = fullfile('PreProcess', FileGroup, 'R');
        SaveImage(rgb_prol_a, 'A3_030108_R_10X_', i, '-BrdU-a-Cutoff.jpg', Dir, 'JPEG');
        SaveImage(rgb_prol_d, 'A3_030108_R_10X_', i, '-BrdU-d-Cutoff.jpg', Dir, 'JPEG');
        SaveImage(rgb_prol_s, 'A3_030108_R_10X_', i, '-BrdU-s-Cutoff.jpg', Dir, 'JPEG');
        
        [int2str(i-n_start+1) ' of ' int2str(n_end-n_start+1) ' for PreR, time used: ' ...
         int2str(round(toc)) 'Sec.']
      end;

      %%%%%%%%%%%%%%%
      % Part III: Ana
      %%%%%%%%%%%%%%@
      if fAna,
        [lfa,nfa] = bwlabel(i_prol_a);
        stats_a = regionprops(lfa,'Area', 'Centroid','PixelIdxList');
        [lfd,nfd] = bwlabel(i_prol_d);
        stats_d = regionprops(lfd,'Area', 'Centroid','PixelIdxList');
        DataOut.N_Of_ProlCell_a = nfa;
        DataOut.L_ProlCell_a = lfa;
        DataOut.Stats_ProlCell_a = stats_a;

        DataOut.N_Of_ProlCell_d = nfd;
        DataOut.L_ProlCell_d = lfd;
        DataOut.Stats_ProlCell_d = stats_d;
        
        DataOut.ProlRate_a = DataOut.N_Of_ProlCell_a/DataOut.N_Of_Cell;
        DataOut.ProlRate_d = DataOut.N_Of_ProlCell_d/DataOut.N_Of_Cell;

        OutputDataFileName = ...
            fullfile('Ana', FileGroup, ['TotDataOut' int2str(i)]);
        save(OutputDataFileName, 'DataOut');
      end;


      [int2str(i-n_start+1) ' of ' int2str(n_end-n_start+1) ' , time used: ' ...
       int2str(round(toc)) 'Sec.']
    end;
    t_end = cputime;
    t_used = t_end-t_start



